#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// by Sylvian Vervoort from S and C, a variation of RSI for divergence trading.
    /// </summary>
    [Description("by Sylvian Vervoort from S and C, a variation of RSI for divergence trading.")]
    public class ARSI : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int period = 14; // Default setting for Period
        // User defined variables (add any user defined variables below)
		
		private	DataSeries		roc_up;
		private DataSeries		roc_up_move;
		private DataSeries		roc_dn_move;
		private DataSeries		dn_count;
		private DataSeries		up_count;
		
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "ARSIPlot"));
            Add(new Line(Color.FromKnownColor(KnownColor.DarkOliveGreen), 70, "OverBought"));
            Add(new Line(Color.FromKnownColor(KnownColor.Maroon), 30, "OverSold"));
            CalculateOnBarClose	= false;
            Overlay				= false;
            PriceTypeSupported	= true;
			
			roc_up = new DataSeries(this);
			roc_up_move = new DataSeries(this);
			roc_dn_move = new DataSeries(this);
			dn_count = new DataSeries(this);
			up_count = new DataSeries(this);
			
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if(CurrentBar <= Period) return;
			
			roc_up.Set(ROC(Input, 1)[0] >= 0 ? 1 : 0);
			up_count.Set(SUM(roc_up ,Period)[0]);
			
			dn_count.Set(Period - up_count[0]);
			
			roc_up_move.Set(ROC(Input, 1)[0] >= 0 ? ROC(Input,1)[0] : 0);
			double up_move = (double) EMA(roc_up_move, (int) (up_count[1] * 2 - 1))[0];
			
			roc_dn_move.Set(ROC(Input, 1)[0] < 0 ? Math.Abs(ROC(Input, 1)[0]) : 0);
			double dn_move = EMA(roc_dn_move, (int) (up_count[1] * 2 - 1))[0];
            
			double rs = up_move / dn_move;
		
			ARSIPlot.Set(100 - (100 / (1 + rs)));
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries ARSIPlot
        {
            get { return Values[0]; }
        }

        [Description("Avg prd")]
        [Category("Parameters")]
        public int Period
        {
            get { return period; }
            set { period = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private ARSI[] cacheARSI = null;

        private static ARSI checkARSI = new ARSI();

        /// <summary>
        /// by Sylvian Vervoort from S and C, a variation of RSI for divergence trading.
        /// </summary>
        /// <returns></returns>
        public ARSI ARSI(int period)
        {
            return ARSI(Input, period);
        }

        /// <summary>
        /// by Sylvian Vervoort from S and C, a variation of RSI for divergence trading.
        /// </summary>
        /// <returns></returns>
        public ARSI ARSI(Data.IDataSeries input, int period)
        {
            checkARSI.Period = period;
            period = checkARSI.Period;

            if (cacheARSI != null)
                for (int idx = 0; idx < cacheARSI.Length; idx++)
                    if (cacheARSI[idx].Period == period && cacheARSI[idx].EqualsInput(input))
                        return cacheARSI[idx];

            ARSI indicator = new ARSI();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Period = period;
            indicator.SetUp();

            ARSI[] tmp = new ARSI[cacheARSI == null ? 1 : cacheARSI.Length + 1];
            if (cacheARSI != null)
                cacheARSI.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheARSI = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// by Sylvian Vervoort from S and C, a variation of RSI for divergence trading.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ARSI ARSI(int period)
        {
            return _indicator.ARSI(Input, period);
        }

        /// <summary>
        /// by Sylvian Vervoort from S and C, a variation of RSI for divergence trading.
        /// </summary>
        /// <returns></returns>
        public Indicator.ARSI ARSI(Data.IDataSeries input, int period)
        {
            return _indicator.ARSI(input, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// by Sylvian Vervoort from S and C, a variation of RSI for divergence trading.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ARSI ARSI(int period)
        {
            return _indicator.ARSI(Input, period);
        }

        /// <summary>
        /// by Sylvian Vervoort from S and C, a variation of RSI for divergence trading.
        /// </summary>
        /// <returns></returns>
        public Indicator.ARSI ARSI(Data.IDataSeries input, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.ARSI(input, period);
        }

    }
}
#endregion
